;******************************************************************************
;*                                                                            *
;* Guillermito particles v0.1 - August 2003 - http://www.guillermito2.net     *
;*                   [copyleft and open source]                               *
;*                                                                            *
;* Some kind of organic-like flocking particles.                              * 
;* Left click for options, right click to move the window.                    *
;* Tested under Win98, Win2000 and WinXP.                                     *
;*                                                                            *
;* The algorithm for the movement of each particle is as follows:             *
;*                                                                            *
;* - start with some kind of pattern (the only moment when pseudo-random      *
;*   routine is called).                                                      *
;* - look in a rectangle (size defined by "detection distance" in the         *
;*   options) around itself, calculate the center of mass of surrounding      * 
;*   particles, including itself.                                             *
;* - if there are less than a certain amount of particles in the vicinity     *
;*   (number defined by "repulsion limit" in the options), than go towards    *
;*   the center of mass with a certain attraction power (defined by "force"   *
;*   in the options - does not depend in the number of particles around).     *
;* - if there are more particles than the defined amount, then fly away from  *
;*   this center of mass.                                                     *
;* - after a number of cycles (defined by "repulsion every" in the options),  *
;*   stir up things a little bit by going through a 50 cycles long total      *
;*   repulsion moment.                                                        *
;*                                                                            *
;* That's it. It looks a little bit like the boids algorithm, but you don't   *
;* have an adjustment of speed of individual particles to match the global    *
;* speed of surrounding neighbours, that's why they don't look like a flock   *
;* of birds or a school of fishes, but more like flies or mosquitoes high on  *
;* drugs. Basically, with the default options, they tend to flock together    *
;* after some time into one big blob moving randomly, and it looks very       *
;* organic. The "repulsion in case of too many neighbours" part of the        *
;* algorithm avoids that hundreds of particles finally concentrate into a     *
;* single pixel. If you play with parameters, especially when "force" is      *
;* stronger, you will see that the global movement of particles may no more   *
;* look natural: for example, the fact that each particle looks in a          *
;* rectangle around it and not a circle (it's just a way to speed up          *
;* calculations - each particle looks at every other one, so it's a routine   *
;* called [particle_number^2] times for each cycle) may become very obvious.  *
;* The other options are mainly cosmetic, or to test the effect on slow       *
;* computers by reducing the number of calculations.                          *
;*                                                                            *
;* The main inspiration for this program was the screen saver "Particle Fire" *
;* which is absolutely fascinating to watch, and programs using the boids     *
;* algorithm, my favorite being another screen saver with 3D OpenGL thingies  *
;* flying around, called "Flocks". Both of these programs are small and very  *
;* efficiently coded (they work smoothly on my old junkyard computer), which  *
;* is important, especially for an assembler coder like me, in these days of  *
;* overbloated pieces of software that do in five megabytes what could be     *
;* done in 20 Kb. Just rambling.                                              *
;*                                                                            *
;* I like this kind of algorithm because it can create complex, chaotic and   *
;* basically unpredictable group patterns, although we start with very simple *
;* individual-based rules. A little bit like cellular automata, or Conway's   *
;* Game of Life. And curiously, they imitate very well some behavior of       *
;* living organisms (in this case, mosquitoes who sucked hallucinogenic       *
;* mushrooms).                                                                *
;*                                                                            *
;* I suspect there is some room for optimization in term of speed (size is    *
;* not really a problem), especially in the core loop. Maybe later I will     *
;* work on that. Well. Probably not.                                          *
;*                                                                            *
;* Compile ressource: gorc.exe /r gparticles.rc                               *
;*          Assemble: tasm32 /ml /m3 /z /t gparticles                         *
;*              Link: tlink32 -Tpe -aa gparticles,,,import32,,gparticles.res  *
;*          Compress: upx -9 gparticles.exe                                   *
;*                                                                            *
;******************************************************************************

callW macro x
extrn x:PROC
call x
endm

.486
.model flat

.data

;****************** structures *************************

wndclass:
clsStyle                  dd 4003h
clsLpfnWndProc            dd ?
clsCbClsExtra             dd 0
clsCbWndExtra             dd 0
clsHInstance              dd ?
clsHIcon                  dd ?
clsHCursor                dd 0
clsHbrBackground          dd 0
clsLpszMenuName           dd 0
clsLpszClassName          dd ?

msg:
msHWND                    dd ?
msMESSAGE                 dd ?
msWPARAM                  dd ?
msLPARAM                  dd ?
msTIME                    dd ?
msPT                      dd ?
protege                   dd ?

;******************** stuff for the windows ***********************

nom_fenetre               db "Guillermito's Particles",0
handle_wd                 dd ?
adresse_retour            dd ?
adresse_retour2           dd ?
dlgname                   db "Options",0
IconName                  db "caticon",0
desktop_height            dd ?
desktop_width             dd ?
y_top                     dd ?
y_bottom                  dd ?
x_left                    dd ?
x_right                   dd ?
theDC                     dd ?

;************ memory organization **********************

buf_bitmap                dd ?            ;size of bitmap pixels buffer
buf_pixels                dd ?            ;size of particle table
mem_header                dd ?            ;offset of buffer for bitmap header + palette
mem_bitmap                dd ?            ;offset of buffer for bitmap pixels
mem_table                 dd ?            ;offset of buffer for particles structures
isolation                 equ 5000        ;to protect from weird pixels

;************ tmp stuff that changes all the time  ************

rnd_seed                  dd 0f00df00dh
tmpy                      dd ?
tmpx                      dd ?
centrey                   dd ?
centrex                   dd ?
centrenb                  dd ?

;**************** variable stuff not worth interacting with *****************

speed_limit               equ 64          ;arbitrary value
explosion_duration        equ 50          ;duration of the short repulsion moment
  
;********** stuff that can be changed interactively ********************

dialog_already_open       dd 0
dialog_smooth             dd 12           ;11 12 13
  degrade_decrement       dd 4            ;less = slow smooth / more = rapid smooth
dialog_size_particle      dd 22           ;21 22 23
dialog_how_to_start       dd 34           ;31 32 33 34
dialog_window_size        dd 41           ;41 42 43 44 45
  width                   dd 400          ;width 
  height                  dd 300          ;height
  width_x_height          dd ?            ;width*height
dialog_zoom_level         dd 111          ;111 112 113
  zoom_level              dd 0            ;0 = same / 1 = 2x / 2 = 4x
dialog_particle_number    dd 52           ;51 52 53 
  nb_particles            dd 300          ;how many particles
dialog_detection_distance dd 62           ;61 62 63
  detection_distance      dd 200000h      ;format XXXX,XXXX so here 32 pixels 
dialog_attraction_power   dd 71           ;71 72 73
  attraction_power        dd 8            ;the higher, the less attraction
dialog_explosion_every    dd 81           ;81 82 83 84
  explosion_every         dd 2000         ;once in a while, total repulsion  
  compteur                dd 0            ;cycle counter
dialog_repulsion_limit    dd 92           ;91 92 93 94
  repulsion_limit         dd 50           ;if X particles too close, repulsion
dialog_color              dd 106          ;101 102 103 104 105 105

.code

CODE_POETRY:

;******************************************************************************
;*********************  SET UP EVERYTHING  ************************************
;******************************************************************************

;---------- change random seed ---------------

callW GetTickCount
bswap eax
xor rnd_seed, eax

;--------- prepare memory buffers ---------------------

call setup_memory

;--------- register wndclass ---------------------

push 0
callW GetModuleHandleA

mov clsHInstance, eax
mov eax, offset wndproc
mov clsLpfnWndProc, eax
mov clsLpszClassName, offset nom_fenetre

push offset IconName
push clsHInstance
callW LoadIconA
mov clsHIcon,eax

push offset wndclass
callW RegisterClassA

;---------- create big whole desktop window  -----------------------

call get_desktop_size

xor eax, eax
push eax
push clsHInstance
push eax
push eax
push desktop_height
push desktop_width
push 0                                  ;y
push 0                                  ;x
push 80000000h                          ;WS_POPUP
mov eax, offset nom_fenetre
push eax
push eax
push 0                                  ;extra style
callW CreateWindowExA
mov handle_wd, eax

;---------- but do not show the whole thing  -----------------------

call define_windows_mask

push 1
push handle_wd
callW ShowWindow

;---------- the message loop ---------------

msg_loop:

xor eax, eax
push 1
push eax
push eax
push eax
push offset msg
callW PeekMessageA
test eax, eax
jnz process_messages

;******************************************************************************
;*********************  ANIMATION LOOP  ***************************************
;******************************************************************************

mov ecx, nb_particles                 ;process all particles in this loop
mov esi, mem_table                    ;esi on particle table

inc compteur
mov eax, explosion_every
add eax, explosion_duration
cmp compteur, eax                     ;if the repulsion moment is over
jne calculate_particles_new_positions
mov compteur, 0                       ;then we reset the counter

calculate_particles_new_positions:

;----------- look around the particle ----------------
;Reminder: one particle is defined by this structure:
;0  dd composante Y speed 
;4  dd composante X speed
;8  dd coordonnee Y
;12 dd coordonnee X

mov centrenb, 0                       ;reset all counters
mov centrey, 0
mov centrex, 0

mov eax, [esi+8]                      ;get Y of the current particle
mov tmpy, eax
mov eax, [esi+12]                     ;get X of the current particle
mov tmpx, eax

pusha

mov ecx, nb_particles
mov esi, mem_table              

    look_around:
    mov eax, [esi+8]                  ;get Y of all particles
    mov ebx, [esi+12]                 ;get X of all particles

    mov edx, eax
    sub edx, tmpy                     ;check Y distance
    jns pasnegy                       ;if negative value
    neg edx                           ;then put it positive
    pasnegy:
    cmp edx, detection_distance       ;is it close enough in Y?
    ja next_particle                  ;too far away => leave

    mov edx, ebx
    sub edx, tmpx                     ;check X distance
    jns pasnegx                       ;if negative value
    neg edx                           ;then put it positive
    pasnegx:
    cmp edx, detection_distance       ;is it close enough in X?
    ja next_particle                  ;too far away => leave
    shr eax, 8                        ;to avoid overflowing if too many particles
    add centrey, eax                  ;sum all the Y of surrounding particles
    shr ebx, 8                        ;to avoid overflowing if too many particles
    add centrex, ebx                  ;sum all the X of surrounding particles
    inc centrenb                      ;remember how many to calculate average later
        
    next_particle:
    add esi, 16                       ;go to next particle in table
    loop look_around

popa

;----- calculate center of mass of surrounding particles -------------

xor edx, edx
mov eax, centrey
div centrenb                          ;calculate average Y
shl eax, 8                            ;no more overflow protection needed now
mov centrey, eax
xor edx, edx
mov eax, centrex
div centrenb                          ;calculate average X
shl eax, 8                            ;no more overflow protection needed now
mov centrex, eax

;-------------- save speed in case it will be too high ------------

push dword ptr [esi]                  ;Y speed
push dword ptr [esi+4]                ;X speed

;--------- is it time for some stirring? ------------------

mov eax, compteur
cmp eax, explosion_every              ;time for total repulsion?
jb normal_way                         ;no
jmp repulsion                         ;yes

normal_way:

;-------- if there are too many particles close, they repulse each other ----------

mov eax, repulsion_limit
cmp centrenb, eax
jb attraction

;-------- REPULSION: get the particle to fly away from the center of mass --------------

repulsion:
push ecx
mov eax, [esi+8]                      ;get Y
sub eax, centrey                      ;get distance from center of mass
mov ecx, attraction_power
sar eax, cl                           ;divide a bit           ;
add [esi], eax                        ;add this vector to Y speed
 
mov eax, [esi+12]                     ;get X
sub eax, centrex                      ;get distance from center of mass
sar eax, cl                           ;divide a bit   
add [esi+4], eax                      ;add this vector to X speed
pop ecx
jmp no_attraction

;------ ATTRACTION: get the particle heading towards the center of mass --------------

attraction:
push ecx
mov eax, [esi+8]                      ;get Y
sub eax, centrey                      ;get distance from center of mass
mov ecx, attraction_power
sar eax, cl                           ;divide a bit           ;
sub [esi], eax                        ;add this vector to Y speed
 
mov eax, [esi+12]                     ;get X
sub eax, centrex                      ;get distance from center of mass
sar eax, cl                           ;divide a bit   
sub [esi+4], eax                      ;add this vector to X speed
pop ecx
no_attraction:

;--------------- check speed limit -------------------

mov eax, [esi]                        ;get Y speed
sar eax, 14                           ;divide a bit so we won't overflow eax
imul eax, eax                         ;squared so always positive
cmp eax, speed_limit                  ;compare with limit
jb speed_Y_ok
mov eax, [esp+4]                      ;get back the saved old speed from the stack
mov [esi], eax                        ;and set it back
speed_Y_ok:
mov eax, [esi+4]                      ;get Y speed
sar eax, 14                           ;divide a bit so we won't overflow eax
imul eax, eax                         ;squared so always positive
cmp eax, speed_limit                  ;compare with limit
jb speed_X_ok
mov eax, [esp]                        ;get back the saved old speed from the stack
mov [esi+4], eax                      ;and set it back
speed_X_ok:
pop eax                               ;empty stack
pop eax

;-------- calculate Y new position ---------------

mov eax, [esi]                        ;load Y speed
add [esi+8], eax                      ;add Y speed to Y coordinate 

;----------- check if it's not outside the window -------------

xor eax, eax                          ;check minimum Y
cmp [esi+8], eax                      ;outside?
jb neg_y_speed                        ;yes => inverse the speed
mov eax, height                       ;no => check maximum Y
shl eax, 16                           ;set it in similar fixed point format
cmp [esi+8], eax                      ;outside?
ja neg_y_speed                        ;yes => inverse the speed
jmp no_out_y                          ;no => get out
neg_y_speed:
neg dword ptr [esi]                   ;inverse Y speed
mov eax, [esi]                        ;and put back the particle
add [esi+8], eax                      ;inside the limits
no_out_y:

;-------- calculate X new position ---------------

mov eax, [esi+4]                      ;load X speed
add [esi+12], eax                     ;add X speed to X coordinate

;----------- check if it's not outside -------------

xor eax, eax                          ;check minimum X
cmp [esi+12], eax                     ;outside?
jb neg_x_speed                        ;yes => inverse the speed
mov eax, width                        ;no => check maximum X
shl eax, 16                           ;set it in similar fixed point format
cmp [esi+12], eax                     ;outside?
ja neg_x_speed                        ;yes => inverse the speed
jmp no_out_x                          ;no => get out
neg_x_speed:
neg dword ptr [esi+4]                 ;inverse X speed
mov eax, [esi+4]                      ;and put back the particle
add [esi+12], eax                     ;inside the limits
no_out_x:

;-------------- transform x,y into a bitmap offset ----------------

mov eax, [esi+8]                      ;load Y coordinate
sar eax, 16                           ;remove decimal part
mov ebx, [esi+12]                     ;load X coordinate
sar ebx, 16                           ;remove decimal part

mov edx, width
imul eax, edx                         ;Y * width in eax
add eax, ebx                          ;add X to that
add eax, mem_bitmap                   ;and the offset of the bitmap buffer
mov edi, eax                          ;all of this in edi
mov eax, 0ffffffffh                   ;color at maximum

;------- verif that particle isn't outside mem buffer and plot it -------------

cmp edi, mem_bitmap                   ;below the buffer?
jb pas_stosb                          ;yes => no plot
mov edx, mem_bitmap                   ;start of bitmap buffer
add edx, width_x_height               ;size of bitmap buffer
add edx, 100                          ;just in case   
cmp edi, edx                          ;above the buffer?
ja pas_stosb                          ;yes => no plot

;----------- plot the particle -----------------------

cmp dialog_size_particle, 21          ;small particle
je small
cmp dialog_size_particle, 22          ;medium particle
je medium
cmp dialog_size_particle, 23          ;big particle
je big

small:
stosb                                 ;  *
jmp end_loopaga

medium:
stosw                                 ;  **
add edi, width                        ;  **
sub edi, 2
stosw
jmp end_loopaga

big:
stosw                                 ;  **
add edi, width                        ; ****
sub edi, 3                            ; ****
stosd                                 ;  **
add edi, width
sub edi, 4
stosd
add edi, width
sub edi, 3
stosw

pas_stosb:

end_loopaga:
add esi, 16                           ;go to next pixel structure
dec ecx
jnz calculate_particles_new_positions

;----------------- smooth by averaging pixels ----------------------

mov ecx, width_x_height               ;do that for the whole bitmap
mov ebp, width
mov esi, mem_bitmap                   ;we start here
add ecx, ebp                          ;add one line on top for bad pixels
add ecx, ebp                          ;another
add ecx, ebp                          ;another
xor eax, eax

smooth:
mov al, [esi-1]                       ;pixel at its left side
add al, [esi-ebp]                     ;pixel above it
adc ah, 0                             ;add carry to ah
add al, [esi+1]                       ;pixel at its right side
adc ah, 0                             ;add carry to ah
add al, [esi+ebp]                     ;pixel under it
adc ah,0                              ;add carry to ah
shr eax, 2                            ;average by dividing by 4
cmp eax, degrade_decrement            ;already black?
jb already_black                      ;yes => don't do anything
sub eax, degrade_decrement            ;no => decrease its color
already_black:
mov [esi], al                         ;plot the new colored pixel
do_not_calculate:
inc esi
loop smooth

;-------- get DC of the window -----------------

push handle_wd
callW GetDC                           ;get device context
mov theDC, eax

;------- bitmap in memory => visible window ---------------

xor ecx, ecx
mov ebx, height
mov edx, width
push 0CC0020h                         ;SRCCOPY=CC0020h
push ecx                              ;DIB_RGB_COLORS=0
push mem_header
push mem_bitmap
push ebx                              ;height bitmap source
push edx                              ;width bitmap source
push ecx                              ;y of bitmap source
push ecx                              ;x of bitmap source
  mov ecx, zoom_level
  shl ebx, cl
push ebx                              ;height destination rectangle
  shl edx, cl
push edx                              ;width destination rectangle
push y_top                            ;y of destination rectangle
push x_left                           ;x of destination rectangle
push eax                              ;DC of window
callW StretchDIBits                   ;no stretch because same size

push theDC
push handle_wd
callW ReleaseDC

jmp msg_loop

;******************************************************************************
;**********************  WINDOW PROCEDURE  ************************************
;******************************************************************************

;-------------- see if window is closed -------------------

process_messages:
cmp msMESSAGE, 12h                    ;WM_QUIT equ 0012h
je end_loop

push offset msg
callW TranslateMessage
push offset msg
callW DispatchMessageA
jmp msg_loop

;--------------------- quit ----------------------

end_loop:
push mem_header
callW LocalFree                       ;free memory

kill:
push -1
callW ExitProcess                     ;and leave

;--------------- window proc ---------------------

wndproc:
pop adresse_retour
cmp dword ptr [esp+4], 2              ;WM_DESTROY?
jne suite                             ;no => continue

push 0
callW PostQuitMessage                 ;yes => quit
xor eax, eax

suite:
cmp dword ptr [esp+4], 0A4h           ;right button click?
jne end_wndproc                       ;no => end

cmp dialog_already_open, 1            ;there's already a dialog window?
je end_wndproc                        ;yes => end
mov dialog_already_open, 1            ;remember it

push 0                                ;initialization value 
push offset dlgproc                   ;where the dialog window procedure is
push handle_wd                        ;handle of window
push offset dlgname                   ;name of the dialog
push clsHInstance                     ;module handle
callW CreateDialogParamA              ;create a modeless dialog: does not block

push 5                                ;SW_SHOW 
push eax                              ;handle_wd
callW ShowWindow                      ;and show it

end_wndproc:
callW DefWindowProcA                  ;call default proc with everything on the stack
push adresse_retour                   ;put back return offset on stack
mov eax, 2                            ;HTCAPTION so right click moves the window
ret

;************************************************************************
;*******************  DIALOG PROCEDURE  *********************************
;************************************************************************

dlgproc:

pusha

push ebp
mov ebp, esp

mov ebx, [ebp+8+20h]
mov eax, [ebp+12+20h]
mov edx, [esp+16+20h]

;-------------- if the dialog is closed --------------

cmp eax, 10h
jne no_end
push 0
push ebx
callW EndDialog
mov dialog_already_open, 0            ;remember it
jmp end_dlgproc_return_1

;--------------- if the dialog is started --------------

no_end:
cmp eax, 110h                         ;WM_INITDIALOG
jne suitez

set_all_buttons:

push dialog_smooth
push 13
push 11
push ebx
callW CheckRadioButton

push dialog_size_particle
push 23
push 21
push ebx
callW CheckRadioButton

push dialog_how_to_start
push 34
push 31
push ebx
callW CheckRadioButton

push dialog_window_size
push 44
push 41
push ebx
callW CheckRadioButton

push dialog_zoom_level
push 113
push 111
push ebx
callW CheckRadioButton

push dialog_particle_number
push 53
push 51
push ebx
callW CheckRadioButton

push dialog_detection_distance
push 63
push 61
push ebx
callW CheckRadioButton

push dialog_attraction_power
push 73
push 71
push ebx
callW CheckRadioButton

push dialog_explosion_every
push 83
push 81
push ebx
callW CheckRadioButton

push dialog_repulsion_limit
push 94
push 91
push ebx
callW CheckRadioButton

push dialog_color
push 106
push 101
push ebx
callW CheckRadioButton

jmp end_dlgproc_return_1  

;-------------- if there is a command sent to the dialog --------------

suitez:
cmp eax, 111h                         ;WM_COMMAND  
jne end_dlgproc_return_0  

;--------------------- check smooth ---------------

check_button:
cmp edx, 11
jne check_next
mov dialog_smooth, edx
mov degrade_decrement, 8
jmp set_button_for_smooth

check_next:
cmp edx, 12
jne check_next1a
mov dialog_smooth, edx
mov degrade_decrement, 4
jmp set_button_for_smooth

check_next1a:
cmp edx, 13
jne check_next2
mov dialog_smooth, edx
mov degrade_decrement, 1
jmp set_button_for_smooth

;-------------- check size particles ----------------

check_next2:
cmp edx, 21
jne check_next3
mov dialog_size_particle, edx
jmp set_button_for_dialog_size_particle

check_next3:
cmp edx, 22
jne check_next4
mov dialog_size_particle, edx
jmp set_button_for_dialog_size_particle

check_next4:
cmp edx, 23
jne check_next5
mov dialog_size_particle, edx
jmp set_button_for_dialog_size_particle

;--------------- check initialization -----------------

check_next5:
cmp edx, dialog_how_to_start          ;to avoid restarting if same button is clicked
je end_dlgproc_return_0

cmp edx, 31
jne check_next6
mov dialog_how_to_start, edx
jmp set_button_for_start

check_next6:
cmp edx, 32
jne check_next7
mov dialog_how_to_start, edx
jmp set_button_for_start

check_next7:
cmp edx, 33
jne check_next8
mov dialog_how_to_start, edx
jmp set_button_for_start

check_next8:
cmp edx, 34
jne check_next9
mov dialog_how_to_start, edx
jmp set_button_for_start

;--------------- check window size -----------------

check_next9:
cmp edx, dialog_window_size           ;to avoid restarting if same button is clicked
je end_dlgproc_return_0

cmp edx, 41
jne check_next11
mov dialog_window_size, edx
mov width, 400
mov height, 300
mov ecx, zoom_level
shr width, cl
shr height, cl
jmp set_button_for_dialog_window_size

check_next11:
cmp edx, 42
jne check_next12
mov dialog_window_size, edx
mov width, 640
mov height, 480
mov ecx, zoom_level
shr width, cl
shr height, cl
jmp set_button_for_dialog_window_size

check_next12:
cmp edx, 43
jne check_next13
mov dialog_window_size, edx
mov width, 800
mov height, 600
mov ecx, zoom_level
shr width, cl
shr height, cl
jmp set_button_for_dialog_window_size

check_next13:
cmp edx, 44
jne check_next13a
mov dialog_window_size, edx
mov eax, desktop_width
mov width, eax
mov eax, desktop_height
mov height, eax
mov ecx, zoom_level
shr width, cl
shr height, cl
jmp set_button_for_dialog_window_size

;---------------- check zoom level ------------------

check_next13a:
cmp edx, dialog_zoom_level            ;to avoid restarting if same button is clicked
je end_dlgproc_return_0

cmp edx, 111
jne check_next13b
mov dialog_zoom_level, edx

mov ecx, zoom_level
shl width, cl
shl height, cl

mov ecx, 0
mov zoom_level, ecx
shr width, cl
shr height, cl
jmp set_button_for_zoom_level

check_next13b:
cmp edx, 112
jne check_next13c
mov dialog_zoom_level, edx

mov ecx, zoom_level
shl width, cl
shl height, cl

mov ecx, 1
mov zoom_level, ecx
shr width, cl
shr height, cl
jmp set_button_for_zoom_level

check_next13c:
cmp edx, 113
jne check_next14
mov dialog_zoom_level, edx

mov ecx, zoom_level
shl width, cl
shl height, cl

mov ecx, 2
mov zoom_level, ecx
shr width, cl
shr height, cl
jmp set_button_for_zoom_level

;---------------- check number of particles ------------------

check_next14:
cmp edx, dialog_particle_number       ;to avoid restarting if same button is clicked
je end_dlgproc_return_0

cmp edx, 51
jne check_next15
mov dialog_particle_number, edx
mov nb_particles, 100
jmp set_button_for_particle_number

check_next15:
cmp edx, 52
jne check_next16
mov dialog_particle_number, edx
mov nb_particles, 300
jmp set_button_for_particle_number

check_next16:
cmp edx, 53
jne check_next17
mov dialog_particle_number, edx
mov nb_particles, 600
jmp set_button_for_particle_number

;---------------- check detection distance ------------------

check_next17:
cmp edx, 61
jne check_next18
mov dialog_detection_distance, edx
mov detection_distance, 100000h
jmp set_button_for_detection_distance

check_next18:
cmp edx, 62
jne check_next19
mov dialog_detection_distance, edx
mov detection_distance, 200000h
jmp set_button_for_detection_distance

check_next19:
cmp edx, 63
jne check_next20
mov dialog_detection_distance, edx
mov detection_distance, 400000h
jmp set_button_for_detection_distance

;---------------- check attraction power ------------------

check_next20:
cmp edx, 71
jne check_next21
mov dialog_attraction_power, edx
mov attraction_power, 8
jmp set_button_for_attraction_power

check_next21:
cmp edx, 72
jne check_next22
mov dialog_attraction_power, edx
mov attraction_power, 6
jmp set_button_for_attraction_power

check_next22:
cmp edx, 73
jne check_next23
mov dialog_attraction_power, edx
mov attraction_power, 4
jmp set_button_for_attraction_power

;---------------- check frequency of stirring ------------------

check_next23:
cmp edx, dialog_explosion_every       ;to avoid restarting if same button is clicked
je end_dlgproc_return_0

cmp edx, 81
jne check_next24
mov dialog_explosion_every, edx
mov explosion_every, 2000
mov compteur, 0
jmp set_button_for_explosion_every

check_next24:
cmp edx, 82
jne check_next25
mov dialog_explosion_every, edx
mov explosion_every, 10000
mov compteur, 0
jmp set_button_for_explosion_every

check_next25:
cmp edx, 83
jne check_next26
mov dialog_explosion_every, edx
mov explosion_every, 65000
mov compteur, 0
jmp set_button_for_explosion_every

check_next26:
cmp edx, 84
jne check_next27
mov eax, explosion_every
mov compteur, eax
jmp end_dlgproc_return_0                       ;special case: no need to setup button

;---------------- check repulsion limit ------------------

check_next27:
cmp edx, 91
jne check_next28
mov dialog_repulsion_limit, edx
mov repulsion_limit, 20
jmp set_button_for_repulsion_limit

check_next28:
cmp edx, 92
jne check_next29
mov dialog_repulsion_limit, edx
mov repulsion_limit, 50
jmp set_button_for_repulsion_limit

check_next29:
cmp edx, 93
jne check_next30
mov dialog_repulsion_limit, edx
mov repulsion_limit, 100
jmp set_button_for_repulsion_limit

check_next30:
cmp edx, 94
jne check_next31
mov dialog_repulsion_limit, edx
mov repulsion_limit, -1
jmp set_button_for_repulsion_limit

;------------ check color ----------------

check_next31:
cmp edx, 101
jne check_next32
mov dialog_color, edx
jmp set_button_for_color

check_next32:
cmp edx, 102
jne check_next33
mov dialog_color, edx
jmp set_button_for_color

check_next33:
cmp edx, 103
jne check_next34
mov dialog_color, edx
jmp set_button_for_color

check_next34:
cmp edx, 104
jne check_next35
mov dialog_color, edx
jmp set_button_for_color

check_next35:
cmp edx, 105
jne check_next36
mov dialog_color, edx
jmp set_button_for_color

check_next36:
cmp edx, 106
jne check_next37
mov dialog_color, edx
jmp set_button_for_color

;--------- check if reset asked ----------------

check_next37:
cmp edx, 1001
jne end_dlgproc_return_0

mov dialog_smooth, 12
mov dialog_size_particle, 22
mov dialog_how_to_start, 34
mov dialog_window_size, 41
mov dialog_zoom_level, 111
mov zoom_level, 0
mov width, 400
mov height, 300
mov dialog_particle_number, 52 
mov nb_particles, 300
mov dialog_detection_distance, 62
mov detection_distance, 200000h 
mov dialog_attraction_power, 71
mov attraction_power, 8
mov dialog_explosion_every, 81
mov explosion_every, 2000
mov compteur, 0
mov dialog_repulsion_limit, 92
mov repulsion_limit, 50
mov dialog_color, 106

pusha
push mem_header
callW LocalFree
call get_desktop_size
call setup_memory
call define_windows_mask
popa

jmp set_all_buttons

;--------- set new buttons and do what's asked ----------------

set_button_for_smooth:
push edx
push 13
push 11
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_dialog_size_particle:
push edx
push 23
push 21
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_start:
mov eax, edx
pusha
call fill_table
popa
push edx
push 34
push 31
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_dialog_window_size:
pusha
push mem_header
callW LocalFree
call get_desktop_size
call setup_memory
call define_windows_mask
popa
push edx
push 44
push 41
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_zoom_level:
pusha
push mem_header
callW LocalFree
call get_desktop_size
call setup_memory
call define_windows_mask
popa
push edx
push 113
push 111
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_particle_number:
pusha
push mem_header
callW LocalFree
call setup_memory
popa
push edx
push 53
push 51
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_detection_distance:
push edx
push 63
push 61
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_attraction_power:
push edx
push 73
push 71
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_explosion_every:
push edx
push 83
push 81
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_repulsion_limit:
push edx
push 94
push 91
push ebx
callW CheckRadioButton
jmp end_dlgproc_return_0

set_button_for_color:
pusha
call set_palette
popa
push edx
push 106
push 101
push ebx
callW CheckRadioButton

end_dlgproc_return_0:
leave
popa
xor eax, eax
ret 16

end_dlgproc_return_1:
leave
popa
xor eax, eax
inc eax
ret 16

;************************************************************************
;*********************  SOME ROUTINES  **********************************
;************************************************************************

;********************* pseudo-random **********************************
; input eax / output eax = pseudo-random number between 0 and eax-1

r_range:

push ecx
push edx
mov ecx,eax
mov eax, 214013h
imul rnd_seed
xor edx, edx
add eax, 2531011h
mov rnd_seed, eax
xor edx,edx
div ecx
mov eax,edx
pop edx
pop ecx
ret

;********************* fill the particle table **********************************

fill_table:

mov edi, mem_table                    ;start of table buffer
mov ecx, nb_particles                 ;reset all particles
mov ebp, height                       ;get height
shl ebp, 15                           ;to fixed point format and divide by 2
mov edx, width                        ;get width
shl edx, 15                           ;to fixed point format and divide by 2

mov eax, dialog_how_to_start          ;how do we fill it?
cmp eax, 31                           ;random?
je fill_table1
cmp eax, 32                           ;horizontal spread?
je fill_table2
cmp eax, 33                           ;vertical spread?
je fill_table3
cmp eax, 34
je fill_table4                        ;explosion?

;------- with everything random ---------------

fill_table1:

mov eax, 20000h                       ;eax = 00 02, 00 00 
call r_range                          ;so y speed random between 0 and 2
mov ebx, 10000h                       ;center it
sub ebx, eax                          ;now y speed between -1 and 1
mov [edi], ebx                        ;save it

mov eax, 20000h                       ;eax = 00 02, 00 00 
call r_range                          ;so x speed random between 0 and 2
mov ebx, 10000h                       ;center it
sub ebx, eax                          ;now x speed between -1 and 1
mov [edi+4], ebx                      ;save it

mov eax, ebp                          ;eax = height/2
shl eax, 1                            ;eax = height
call r_range                          ;Y random between 0 and height
mov [edi+8], eax                      ;save

mov eax, edx                          ;eax = width/2
shl eax, 1                            ;eax = width 
call r_range                          ;X random between 0 and height
mov [edi+12], eax                     ;save

add edi, 16                           ;go to the next particle structure
loop fill_table1
ret

;----------- with a horizontal thing -----------------

fill_table2:

mov eax, 20h                          ;very small random y speed
call r_range
mov ebx, 10h
sub ebx, eax
mov [edi], ebx

mov eax, 20000h                       ;eax = 00 02, 00 00 
call r_range                          ;so x speed random between 0 and 2
mov ebx, 10000h                       ;center it
sub ebx, eax                          ;now x speed between -1 and 1
mov [edi+4], ebx                      ;save it

mov [edi+8], ebp                      ;for all y = height/2
mov [edi+12], edx                     ;for all x = width/2

add edi, 16                           ;go to the next particle structure
loop fill_table2
ret

;----------- with a vertical thing -----------------

fill_table3:

mov eax, 20000h                       ;eax = 00 02, 00 00 
call r_range                          ;so y speed random between 0 and 2
mov ebx, 10000h                       ;center it
sub ebx, eax                          ;now y speed random between -1 and 1
mov [edi], ebx                        ;save it

mov eax, 20h                          ;very small random x speed
call r_range
mov ebx, 10h
sub ebx, eax
mov [edi+4], ebx

mov [edi+8], ebp                      ;for all y = height/2
mov [edi+12], edx                     ;for all x = width/2

add edi, 16                           ;go to the next particle structure
loop fill_table3
ret

;------- with an explosion -------------------

fill_table4:

mov centrey, ecx                      ;use counter as a "random" radian angle value
fild centrey                          ;stack it in coprocessor
fsin                                  ;get sinus
mov eax, 10000h                       ;eax = 00 01, 00 00
call r_range                          ;get a random speed
mov centrey, eax                      ;put in in a tmp memory offset 
fild centrey                          ;stack it in coprocessor
fmul                                  ;multiply with sinus
fistp dword ptr [edi]                 ;destack and save it as y speed 

mov centrey, ecx                      ;use counter as a "random" radian angle value
fild centrey                          ;stack it in coprocessor
fcos                                  ;get cosinus
mov eax, 10000h                       ;eax = 00 01, 00 00
call r_range                          ;get a random speed
mov centrey, eax                      ;put in in a tmp memory offset 
fild centrey                          ;stack it in coprocessor
fmul                                  ;multiply with sinus
fistp dword ptr [edi+4]               ;destack and save it as x speed

mov [edi+8], ebp                      ;for all y = height/2
mov [edi+12], edx                     ;for all x = width/2

add edi, 16                           ;go to the next particle structure
dec ecx
jnz fill_table4
ret

;----------- get desktop size and calculate a few things to center ----------

get_desktop_size:

push 1                                ;SM_CYSCREEN
callW GetSystemMetrics                ;get screen height
mov desktop_height, eax
  mov ebx, height
  mov ecx, zoom_level
  shl ebx, cl
sub eax, ebx                          ;substract height of our window
shr eax, 1                            ;and divide the rest by 2 => centered
mov y_top, eax
add eax, ebx
mov y_bottom, eax

push 0                                ;SM_CXSCREEN
callW GetSystemMetrics                ;get screen width
mov desktop_width, eax
  mov ebx, width
  mov ecx, zoom_level
  shl ebx, cl
sub eax, ebx                          ;substract width of our window
shr eax, 1                            ;and divide the rest by 2 => centered
mov x_left, eax
add eax, ebx
mov x_right, eax
ret

;---------- define the window mask -------------------

define_windows_mask:

push y_bottom                         ;y lower-right
push x_right                          ;x lower-right
push y_top                            ;y upper-left 
push x_left                           ;x upper-left
callW CreateRectRgn                   ;return region handle

push 1                                ;window redraw flag 1=true
push eax                              ;handle to region 
push handle_wd
callW SetWindowRgn
ret

;--------------- setup memory ---------------------

setup_memory:

;--------- memory for bitmap + particle table --------------------

mov eax, width
mov ebx, height
imul eax, ebx                         ;nb of pixels in the bitmap
mov width_x_height, eax               ;save that
add eax, isolation                    ;to avoid bad surprises
mov buf_bitmap, eax                   ;buf_bitmap equ (width*height)+isolation

mov ebx, nb_particles
shl ebx, 4                            ;16 bytes for each particle
add ebx, isolation                    ;to avoid bad surprises
mov buf_pixels, ebx                   ;buf_pixels equ (16*nb_particles)+isolation

buf_header equ (256*4)+40+isolation   ;(256colors*RGB0)+BMPheader+isolation

add eax, ebx
add eax, buf_header                   ;number of bytes to allocate
push eax
push 40h                              ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax                         ;problem?
jz kill                               ;yes => quit
mov mem_header, eax                   ;bitmap header+palette start here
add eax, buf_header
mov mem_bitmap, eax                   ;bitmap pixels start here
add eax, buf_bitmap
mov mem_table, eax                    ;particle table start here

;------------- fill the particle table -----------------

call fill_table

;---- construct BITMAPINFO (BITMAPINFOHEADER + Palette) in memory --------
;Reminder:
;BITMAPINFO:
; BITMAPINFOHEADER: 
;    biSize          dd bmpinfosize   ;biSize (40d)
;    biWidth         dd width         ;biWidth
;    biHeight        dd height        ;biHeight
;    biPlanes        dw 1             ;biPlanes
;    biBitCount      dw 8             ;biBitCount (8=256 colors)
;    biCompression   dd 0             ;biCompression
;    biSizeImage     dd 0             ;biSizeImage
;    biXPelsPerMeter dd 0             ;biXPelsPerMeter
;    biYPelsPerMeter dd 0             ;biYPelsPerMeter
;    biClrUsed       dd 0             ;biClrUsed
;    biClrImportant  dd 0             ;biClrImportant
;PALETTE db 256*4 dup (99h)           ;256 couleurs * (RGB + reserved)

mov edi, mem_header
xor eax, eax
mov al, 40d
stosd
mov eax, width
stosd
mov eax, height
stosd
mov eax, 00080001h
stosd

;------- fill palette (BGR0) with increasing intensities ----------

set_palette:

mov edi, mem_header
add edi, 40d                          ;put edi at the palette start
mov ebp, dialog_color

xor ecx, ecx                          ;use as counter
xor eax, eax                          ;use for blue value
xor ebx, ebx                          ;use for geen value
xor edx, edx                          ;use for red value

fill_palette:
mov [edi], al
mov [edi+1], bl
mov [edi+2], dl

cmp ebp, 101
je fill_palette1                      ;black => white
cmp ebp, 102
je fill_palette2                      ;blue => red => yellow
cmp ebp, 103
je fill_palette3                      ;blue => white
cmp ebp, 104
je fill_palette4                      ;green => yellow
cmp ebp, 105
je fill_palette5                      ;red => white
cmp ebp, 106
je fill_palette6                      ;red => yellow

fill_palette1:
inc eax
inc ebx
inc edx
jmp end_loop_palette

fill_palette2:
cmp ecx, 170
ja green
cmp ecx, 85
ja decb
inc eax
inc eax
jmp green
decb:
dec eax
dec eax
green:
cmp ecx, 170
jb red
inc ebx
inc ebx
inc ebx
red:
cmp ecx, 85
jb nomax
cmp ecx, 168
ja nomax
inc edx
inc edx
inc edx
nomax:
jmp end_loop_palette

fill_palette3:
cmp ecx, 85
ja green1
inc eax
inc eax
green1:
cmp ecx, 85
jb red1
cmp ecx, 170
ja red1
inc ebx
inc ebx
red1:
cmp ecx, 170
jb nomax1
inc edx
inc edx
nomax1:
jmp end_loop_palette

fill_palette4:
inc ebx
red2:
cmp ecx, 170
jb nomax2
inc edx
inc edx
inc edx
nomax2:
jmp end_loop_palette

fill_palette5:
cmp ecx, 170
jb green3
inc eax
inc eax
inc eax
green3:
cmp ecx, 170
jb red3
inc ebx
inc ebx
inc ebx
red3:
inc edx
nomax3:
jmp end_loop_palette

fill_palette6:
cmp ecx, 128
jb red4
inc ebx
inc ebx
red4:
cmp ecx, 126
ja nomax4
inc edx
inc edx
nomax4:

end_loop_palette:
add edi, 4
inc ecx
cmp ecx, 255
jne fill_palette

ret

end CODE_POETRY
